#include <Bounce2.h>
#include <SoftwareSerial.h>
#include <AccelStepper.h>

// Define a stepper and the pins it will use
AccelStepper stepper(1,7,6); // define (Mode,Pul,Dir)

#define link 5 //pin for the link
#define cal 4  //pin for the calibration

//fill the tank 24.5cm from the bottom
//motor connections: black A+ | green A- | red B+ | blue B-
//default wedge depth for data collection 68mm
//DEFAULT CONFIG: Wamp 75mm | Z0 68mm


//WAVE PARAMETRES
long z0=68;         //set the wedge starting point [mm]
long wAmp=75;       //set the pp wave amplitude [mm]
float gRatio=16.66; //set the rot to linear gear ratio of N step for 1mm
float T=0.9;        //set wave period

long mS=0; 
long t=0;
long acc=0;

Bounce2::Button buttonTop = Bounce2::Button();

void setup(){
  Serial.begin(9600);

  //setting up Arduino latching link
  pinMode(link,OUTPUT);
  digitalWrite(link,LOW);

  //setting up the inizialization button
  buttonTop.attach(cal, INPUT_PULLUP ); //pin 4 for the switch
  buttonTop.interval(5); 
  buttonTop.setPressedState(LOW);
  
  //PARAMETERS CALCULATION
  z0=(long)z0*gRatio;
  
  //ramp wave
  //mS=(long)((wAmp*2)/T)*gRatio;
  //acc=80000;

  //sinusoidal wave
  mS=(long)30000*gRatio;
  acc= (long)(wAmp)/(T*T/256);
  wAmp=(long)(wAmp/2)*gRatio;

  //STEPPER INIZIALIZATION
  stepper.setMaxSpeed(800);
  stepper.setAcceleration(acc);
  delay(3000);
  
  //inizialization
  buttonTop.update();
  while(!buttonTop.pressed()){
    stepper.move(-1);
    stepper.run();
    buttonTop.update();
  }
  //Serial.println("buttonTop");
  delay(1000);
  stepper.move(z0);
  stepper.runToPosition();
  delay(3000);
  
  //stepper configuration
  stepper.setCurrentPosition(0);
  stepper.setMaxSpeed(mS);
  stepper.moveTo(wAmp); 
}

void loop(){
  
    // If at the end of travel go to the other end
    if (stepper.distanceToGo() == 0){
      //period calculation
      //Serial.println(millis()-t);
      //t=millis();
      
      stepper.moveTo(-stepper.currentPosition());

      //LATCHING SIGNAL
      if(stepper.distanceToGo()<0){
        Serial.println(1);
        digitalWrite(link,HIGH);
        }else{
        digitalWrite(link,LOW);
        Serial.println(0);
        }
          
    }   
    stepper.run();
}
